# -*- coding: utf-8 -*-

import os
import sys
import config
import time
import logging
import traceback
# import uno
import datetime
import subprocess
import codecs

from requests.exceptions import ConnectionError
from requests.exceptions import Timeout
from subprocess import CalledProcessError
from subprocess import PIPE
# from com.sun.star.task import ErrorCodeIOException
from exception import DocumentConversionException
# from exception import UnoConnectionException
# from unoconv import DocumentConverter

import lbocr
from lbrest import LBRest
from lbdoc import DocumentConverter
from lbparser import LBParser


def convert_files(args):

    """ Download each file from base and try to extract texts. """
    base = args['metadata']['name']
    file_ext_time = args['metadata']['file_ext_time']
    logger.info("""
                STARTING PROCESS EXECUTION FOR %s -- PID %s
                IN PARENT PROCESS -- PID %s
                """ % (base, os.getpid(), os.getppid())
                )
    if config.PAGE_SIZE > 0:
        logger.info(" - Running in Paged Mode - \n")
    else:
        logger.info(" - Running in non-Paged Mode - \n")
    while True:
        try:
            base_converter = BaseConverter(base, file_ext_time)
            if base_converter.paged_mode():
                base_converter.run_paged_conversion()
            else:
                base_converter.run_conversion()
        except (ConnectionError, Timeout) as e:
            logger.critical("""
                            Could not connect to server! %s.
                            Waiting for %s minutes to run next.
                            """ % (config.REST_URL, str(file_ext_time))
                            )
            time.sleep(datetime.timedelta(minutes=int(file_ext_time)).seconds)
        except Exception as e:
            logger.critical("""
                            Uncaught Exception : %s.
                            Waiting for %s minutes to run next.
                            """ % (traceback.format_exc(), str(file_ext_time))
                            )
            time.sleep(datetime.timedelta(minutes=int(file_ext_time)).seconds)


class BaseConverter():

    def __init__(self, base, file_ext_time):

        """ Create a new BaseConverter object. """
        self.base = base
        self.file_ext_time = file_ext_time
        self.lbrest = LBRest(base)
        self.offset = 0
        if config.PAGE_SIZE <= 0:
            self.files = self.lbrest.get_files()
        else:
            self.files = self.lbrest.get_some_files(config.PAGE_SIZE, self.offset)
        self.passed_files = self.lbrest.get_passed_files()

    def paged_mode(self):
        if config.PAGE_SIZE > 0:
            return True
        else:
            return False

    def is_supported_file(self, id, file_name, file_format, id_doc):

        """ Check if the file is supported by consulting the supported
            files list.

            Keyword Arguments:
            id -- file ID code
            file_name -- file name on computer
            file_format -- file format e.g.: '.doc'

            Return: bool
        """
        if file_format not in config.SUPPORTED_FILES:
            logger.info('File %s , with id %s from base %s is not supported, writing " "' % (file_name, id, self.base))
            self.write_text(id, '', id_doc, file_name)
            return False
        return True

    def run_conversion(self):

        """ Run text extraction for all pending files. """
        # Check if Daemon has stopped
        self.check_ppid()

        # Check OpenOffice.org process
        # self.run_soffice()

        # Get initial time
        ti = datetime.datetime.now()

        for _file in self.files:

            # Check if Daemon has stopped
            self.check_ppid()

            file_name = _file['filename']
            file_format = file_name.split(".")[-1].lower()
            download_url = _file['download']
            id_file = _file['id_file']
            id_doc = _file['id_doc']
            file_size = _file['filesize']

            if id_file not in self.passed_files:
                # continue
                if file_size == 0:
                    self.write_text(id_file, " ", id_doc, file_name)
                    continue
                if not self.is_supported_file(id_file, file_name, file_format, id_doc):
                    continue
                logger.info('Arquivo: base=%s, id_file=%s, nome=%s' % (self.base, id_file, file_name))
                try:
                    local_path = self.lbrest.download(download_url, id_file)
                except Exception, error_msg:
                    self.write_log_error(error_msg, id_file, id_doc, file_name)
                    continue
                text = self.extract_text(id_file, id_doc, file_name,
                                         local_path, file_format)
                os.remove(local_path)
                if text is not None:
                    self.write_text(id_file, text, id_doc, file_name)

        # Get final time
        tf = datetime.datetime.now()
        self.sleep(ti, tf)

    def run_paged_conversion(self):

        """ Run text extraction for all pending files in paged mode """
        # Check if Daemon has stopped
        self.check_ppid()

        # Check OpenOffice.org process
        # self.run_soffice()

        # Get initial time
        ti = datetime.datetime.now()
        page_number = 1
        while self.files:
            logger.info("- - PAGE " + page_number.__str__() + " - -")
            page_number += 1
            for _file in self.files:

                # Check if Daemon has stopped
                self.check_ppid()

                file_name = _file['filename']
                file_format = file_name.split(".")[-1].lower()
                download_url = _file['download']
                id_file = _file['id_file']
                id_doc = _file['id_doc']
                file_size = _file['filesize']

                if id_file not in self.passed_files:
                    # continue
                    if file_size == 0:
                        self.write_text(id_file, " ", id_doc, file_name)
                        continue
                    if not self.is_supported_file(id_file, file_name, file_format, id_doc):
                        continue
                    logger.info('Arquivo: base=%s, id_file=%s, nome=%s' % (self.base, id_file, file_name))
                    try:
                        local_path = self.lbrest.download(download_url, id_file)
                    except Exception, error_msg:
                        self.write_log_error(error_msg, id_file, id_doc, file_name)
                        continue
                    text = self.extract_text(id_file, id_doc, file_name,
                                             local_path, file_format)
                    os.remove(local_path)
                    if text is not None:
                        self.write_text(id_file, text, id_doc, file_name)
            self.offset += config.PAGE_SIZE
            self.files = self.lbrest.get_some_files(config.PAGE_SIZE, self.offset)

        # Get final time
        tf = datetime.datetime.now()
        self.sleep(ti, tf)

    def extract_text(self, id_file, id_doc, file_name,
                     file_path, file_format):

        """ Extract text from file.

            Keyword Arguments:
            id_file -- file ID code
            id_doc -- document ID code
            file_name -- file name on computer
            file_path -- file full path on computer
            file_format -- file format e.g.: '.doc'

            Return: proper extraction method for file type.
        """
        extract_methods = {
            'pdf': self.read_pdf,
            'xml': self.read_textfile,
            'html': self.read_textfile,
            'htm': self.read_textfile,
            'txt': self.read_textfile,
            }

        extract_method = extract_methods.get(file_format, self.read_document)
        return extract_method(id_file, id_doc, file_name,
                              file_path, file_format)

    def write_log_error(self, error_msg, id_file, id_doc, file_name):

        """Record errors occurred. """
        logger.error(error_msg)
        self.lbrest.write_error(id_file, id_doc, file_name, str(error_msg))

    def write_text(self, id_file, text, id_doc, file_name):
        try:
            self.lbrest.write_text(id_file, text)
        except Exception as error_msg:
            self.write_log_error(error_msg, id_file, id_doc, file_name)

    def read_pdf(self, id_file, id_doc, file_name, file_path, file_format):

        """ Get text from pdf file, using pdftotext tool.
            Also, if the pdf file has any images, try to extract any
            text in the image file using OCR tool tesseract-ocr.

            Keyword arguments:
            id_file -- file ID code
            id_doc -- document ID code
            file_name -- file name on computer
            file_path -- file full path on computer
            file_format -- file format e.g.: '.doc'

            Return: string -- extracted text from pdf file
        """
        pdftotext_error = False
        tesseract_error = False
        cmd = [config.PDFTOTEXT_CMD, file_path, '-']
        try:
            process = subprocess.Popen(cmd, stdout=PIPE,
                                       stderr=PIPE, stdin=PIPE)
            file_text, error_msg = process.communicate()
        except Exception as e:
            error_msg = ('pdftotext command %s not working properly: %s'
                         % (str(cmd), e))

        if error_msg:
            #self.write_log_error(error_msg, id_file, id_doc, file_name)
            # self.lbrest.write_error(id_file, id_doc,
            #                         file_name, error_msg)
            pdftotext_error = True
            file_text = file_text.decode('iso-8859-1').encode('utf8')
        else:
            file_text = file_text.decode('iso-8859-1').encode('utf8')

        # '/tmp/<base>-images/file-<id_file>'
        images_dir = '/'.join(['/tmp', self.base + '-images',
                              'file-' + str(id_file)])
        # images = lbocr.extract_pdf_images(file_path, images_dir)
        images = lbocr.pdf2image(file_path, images_dir)

        for image_path in images:
            try:
                text = lbocr.image_to_string(image_path).encode('utf-8')
            except lbocr.TesseractError as e:
                tesseract_error = True
                logger.error('TesseractError: %s' % e)
            except Exception as e:
                tesseract_error = True
                logger.error('TesseractError: %s' % e)
            else:
                image_label = '\n' + '## ' + \
                              image_path.split('/')[-1].split('.')[0] +\
                              ' ##' + '\n'
                file_text = file_text + image_label.encode('utf-8') + text
            finally:
                lbocr.cleanup(image_path)
        if pdftotext_error and tesseract_error:
            self.write_log_error(error_msg, id_file, id_doc, file_name)
            lbocr.cleanup_dirs(images_dir)
            return None
        else:
            lbocr.cleanup_dirs(images_dir)
            return file_text

    def read_textfile(self, id_file, id_doc, file_name,
                      file_path, file_format):

        """ Get text from file.

            Keyword arguments:
            id_file -- file ID code
            id_doc -- document ID code
            file_name -- file name on computer
            file_path -- file full path on computer
            file_format -- file format extension e.g.: '.doc'

            Return:
            text_file -- string
            or
            None if text_file is None
            or
            call lbparser.get_text() method for .htm and .html files
        """
        encoding = self.get_encoding(file_path)
        ext_parser = ['html', 'htm']
        try:
            with codecs.open(file_path, encoding=encoding) as f:
                file_text = f.read()
            if file_format not in ext_parser:
                return file_text or None
            else:
                lbparser = LBParser(file_text)
                return lbparser.get_text()
        except Exception as exception:
            error_msg = 'Erro ao ler arquivo %s : %s' % (file_format,
                                                         exception)
            # logger.error(error_msg)
            self.write_log_error(error_msg, id_file, id_doc, file_name)

    def read_xhtmlfile(self, id_file, id_doc, file_name,
                       file_path, file_format):
        """ Open htm/html file

            Keyword arguments:
            id_file -- file ID code
            id_doc -- document ID code
            file_name -- file name on computer
            file_path -- file full path on computer
            file_format -- file format extension e.g.: '.doc'
        """
        gross_text = self.read(id_file, id_doc, file_name,
                               file_path, file_format)
        try:
            parser = LBParser(gross_text)
        except Exception, e:
            raise e

    def read_document(self, id_file, id_doc, file_name,
                      file_path, file_format):

        """ Read document [doc, docx, odt, rtf]

            Keyword arguments:
            id_file -- file ID code
            id_doc -- document ID code
            file_name -- file name on computer
            file_path -- file full path on computer
            file_format -- file format extension e.g.: '.doc'

            Call DocumentConverter.get_text() method
        """
        try:
            converter = DocumentConverter(file_path, file_name,
                                          file_format, id_file)
            file_text = converter.get_text()
            return file_text
        except Exception as error_msg:
            # logger.error(error_msg)
            self.write_log_error(error_msg, id_file, id_doc, file_name)
            # self.lbrest.write_error(id_file, id_doc, file_name,
            #                         str(error_msg))

        """
        try:
            converter = DocumentConverter()
            file_text = converter.get_file_text(file_path,
                                                file_format)
            return file_text or None
        except UnoConnectionException as exception:
            error_msg= 'UnoConnectionException: %s' % str(exception)
            logger.error(error_msg)
            self.lbrest.write_error(id_file, id_doc, file_name,
                                    error_msg)
            self.run_soffice()
        except DocumentConversionException as exception:
            error_msg= 'DocumentConversionException: %s'
                        % str(exception)
            logger.error(error_msg)
            self.lbrest.write_error(id_file, id_doc, file_name,
                                    error_msg)
            self.run_soffice()
        except ErrorCodeIOException as exception:
            error_msg = 'ErrorCodeIOException : %d' % exception.ErrCode
            logger.error(error_msg)
            self.lbrest.write_error(id_file, id_doc, file_name,
                                    error_msg)
        except Exception as exception:
            error_msg = 'Exception : %s' % exception
            logger.error(error_msg)
            self.lbrest.write_error(id_file, id_doc, file_name,
                                    error_msg)
        """

    def sleep(self, ti, tf):
        """ Calculate interval."""

        execution_time = tf - ti
        _file_ext_time = datetime.timedelta(minutes=self.file_ext_time)

        if execution_time >= _file_ext_time:
            interval = 0
        else:
            interval = _file_ext_time - execution_time
        # interval = _file_ext_time
        if type(interval) is not int:
            interval_minutes = (interval.seconds//60) % 60
            interval_seconds = interval.seconds
        else:
            interval_minutes = interval_seconds = interval

        #logger.info('Finished execution for base %s, will wait for %s minutes' % (self.base, str(interval_minutes)))

        # Sleep interval
        time.sleep(interval_seconds)

    def check_ppid(self):
        """ Stop process if daemon is not running."""

        pid = os.getppid()
        if pid == 1:
            logger.info('STOPPING PROCESS EXECUTION FOR %s' % self.base)
            sys.exit(0)

    def run_soffice(self):

        """ Check if soffice process is running. Case negative,
            tries to run it.
        """
        try:
            subprocess.check_call('ps -ef | grep -v grep | grep soffice.bin',
                                  shell=True, stdout=subprocess.PIPE)
        except CalledProcessError:
            command = [
                'soffice',
                '--accept=socket,host=localhost,port=%s;urp;\
                    StarOffice.Service' % config.DEFAULT_OPENOFFICE_PORT,
                '--headless',
                '--nologo',
                '--nofirststartwizard'
            ]
            subprocess.Popen(command, stdout=subprocess.PIPE)
            time.sleep(5)

    def get_encoding(self, file_path):

        """ Guess file encoding.

            Return: encoding -- string
            or
            None if error
        """
        try:
            # output = subprocess.check_output(['file',
            #    '--mime-encoding', file_path])
            output = subprocess.call(['file', '--mime-encoding', file_path])
        except (CalledProcessError, OSError) as e:
            logger.error('Could not guess file encoding. Error: %s' % e)
            return None

        encoding = None
        if type(output) is str and len(output.split(':')) > 1:
            encoding = output.split(':')[1]
            try:
                with codecs.open('/dev/null', encoding=encoding) as f:
                    pass
            except LookupError:
                encoding = None

        return encoding

logger = logging.getLogger("LBConverter")
