
# -*- coding: utf-8 -*-

import requests
import logging
import config
from time import sleep
import uuid
import datetime
import json
import os


class LBRest():

    def __init__(self, base=None):
        self.base = base

    def create_base_log(self):
        """Creates the database log_lbconverter
        """
        result = False
        data = """{
                "metadata":{
                    "file_ext":false,
                    "idx_exp":false,
                    "idx_exp_url":"",
                    "idx_exp_time":0,
                    "file_ext_time":0,
                    "name":"log_lbconverter",
                    "description":"LightBase - Log de erros do LBConverter",
                    "password":"qqqqqqqq",
                    "color":"#000000"
                    },
                "content":[
                        {"field":
                            {"name":"nm_base",
                            "alias":"Nome da base",
                            "description":"Nome da base",
                            "datatype":"Text",
                            "required":true,
                            "multivalued":false,
                            "indices":["Ordenado"]
                            }
                        },
                        {"field":
                            {"name":"id_doc_orig",
                            "alias":"id_doc_orig",
                            "description":"id do documento que originou o erro.",
                            "datatype":"Integer",
                            "required":true,
                            "multivalued":false,
                            "indices":["Textual","Ordenado"]
                            }
                        },
                        {"field":
                              {"name":"id_file_orig",
                              "alias":"id_file_orig",
                              "description":"ID do arquivo que originou o erro.",
                              "datatype":"Text",
                              "required":true,
                              "multivalued":false,
                              "indices":["Textual","Unico"]
                              }
                        },
                        {"field":
                              {"name":"file_name",
                              "alias":"file_name",
                              "description":"File name",
                              "datatype":"Text",
                              "required":true,
                              "multivalued":false,
                              "indices":["Ordenado"]
                            }
                        },
                        {"field":
                               {"name":"error_msg",
                               "alias":"Mensagem de erro",
                               "description":"Mensagem de erro",
                               "datatype":"Text",
                               "required":true,
                               "multivalued":false,
                               "indices":["Nenhum"]
                               }
                        },
                        {"field":
                               {"name":"dt_error",
                               "alias":"Data do erro",
                               "description":"Data do erro",
                               "datatype":"DateTime",
                               "required":true,
                               "multivalued":false,
                               "indices":["Textual","Ordenado"]
                               }
                        }
                        ]
                }
        """
        logger.info("Criando a base...")
        req = requests.post(config.REST_URL, data = {'json_base': data}, headers=config.HEADERS)
        try:
            req.raise_for_status()
            response = int(req.text)
            if response > 0:
                result = True
        except:
             logger.error("""
                Erro ao tentar criar base. url: %s. Reposta: %s
            """ % (config.REST_URL, req._content))
        return result

    def base_exists(self):
        """Checks if there is the base.
        """
        exists = False
        logger.info("Verificando se a base existe...")
        url = config.REST_URL + "/log_lbconverter"
        req = requests.get(url, headers=config.HEADERS)
        try:
            req.raise_for_status()
            logger.info("A base existe...")
            exists = True
        except:
            if req.status_code == 404:
                logger.info("A base não existe.")
            else:
                logger.error("""
                Erro ao tentar recuperar base. url: %s. Reposta: %s
            """ % (config.REST_URL, req._content))
        return exists

    def get_bases(self):
        """ Get all bases which has to extract file texts
        """
        bases = [ ]
        params = """{
            "select": [
                "name",
                "file_ext_time"
            ],
            "literal": "file_ext is true"
        }"""
        req = requests.get(config.REST_URL, params={'$$':params}, headers=config.HEADERS)
        try:
            req.raise_for_status()
            response = req.json()
            bases = response["results"]
        except:
            logger.error("""
                Erro ao tentar recuperar bases. url: %s. Reposta: %s
            """ % (config.REST_URL, req._content))
        return bases

    def get_some_files(self, quant, offset):
        """ Get 'quant' files which have to be converted to text
            @quant: limit files
            @offset: start index query
            http://127.0.0.1/api/db_reg_anot_teses_arq/file?$$={"select":["*"],"literal":"dt_ext_text is null and id_doc is not null","limit":10,"offset":20,"order_by":{"asc":["id_doc"]}}
        """
        files = [ ]
        req = '{"select":["id_doc","id_file","download","filename","filesize"],"literal":"dt_ext_text is null and id_doc is not null", "limit":'+str(quant)+',"offset":'+str(offset)+',"order_by":{"asc":["id_doc"]}}&result_count=false'
        params = {'$$':req}
        url = config.REST_URL + '/' + self.base + '/file'
        req = requests.get(url, params=params, headers=config.HEADERS)
        try:
            req.raise_for_status()
            response = req.json()
            files = response["results"]
        except:
            logger.error("""
                Erro ao tentar recuperar arquivos da base %s. Reposta: %s
            """ % (self.base, req._content))
        return files


    def get_files(self):
        """ Get all files which have to be converted to text
        """
        files = [ ]
        params = {'$$':'{"select":["id_doc","id_file","download","filename","filesize"],"literal":"dt_ext_text is null and id_doc is not null", "limit": null}&result_count=false'}
        url = config.REST_URL + '/' + self.base + '/file'
        req = requests.get(url, params=params, headers=config.HEADERS)
        try:
            req.raise_for_status()
            response = req.json()
            files = response["results"]
        except:
            logger.error("""
                Erro ao tentar recuperar arquivos da base %s. Reposta: %s
            """ % (self.base, req._content))
        return files

    def write_text(self, id_file, text):
        """ Write extracted text from file to LightBase
        """
        url = config.REST_URL + '/' + self.base + '/file/' + str(id_file) + '/text'
        logger.info('Escrevendo texto em : ' + url)
        data = {'filetext': text}
        req = requests.put(url, data=data, headers=config.HEADERS)
        try:
            req.raise_for_status()
        except:
            if req.status_code != 404:
                logger.error("""
                    Erro ao tentar alterar texto do arquivo %s da base %s. Reposta: %s
                """ % (str(id_file), self.base, req._content))
            else:
                raise Exception(req._content)

    def download(self, url, id_file):
        """Download file from url"""
        #local_filename = '/tmp/' + str(uuid.uuid4())
        #local_filename =  config.OUTPATH + '/' + str(uuid.uuid4())
        local_filename =  config.OUTPATH + '/' + str(id_file)
        #http://stackoverflow.com/questions/16694907/how-to-download-large-file-in-python-with-requests-py
        req = requests.get(url, stream=True, headers=config.HEADERS)
        try:
            req.raise_for_status()
        except:
            msg = """Erro ao tentar baixar arquivo na url %s.
                     Resposta: %s """ % (url, req._content)
            raise Exception(msg)
        if os.path.exists(local_filename):
            os.remove(local_filename)
        if not os.path.exists(config.OUTPATH):
            os.makedirs(config.OUTPATH)
        with open(local_filename, 'wb') as f:
            for chunk in req.iter_content(chunk_size=1024):
                if chunk: # filter out keep-alive new chunks
                    f.write(chunk)
                    f.flush()
        return local_filename

    def write_error(self, id_file, id_doc, file_name, error_msg):
        """ Write errors to LightBase
        """
        error = {
            'nm_base': self.base,
            'id_doc_orig': id_doc,
            'id_file_orig': id_file,
            'file_name': file_name,
            'error_msg': error_msg,
            'dt_error': datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S')
        }
        url = config.REST_URL + '/log_lbconverter/doc'
        data = {'value': json.dumps(error)}
        req = requests.post(url, data=data, headers=config.HEADERS)
        try:
            req.raise_for_status()
        except:
            logger.error("""
                Erro ao tentar escrever erro no Lightbase. Reposta: %s
            """ % req._content)

    def get_passed_files(self):
        """ Get Files that we couldn't get text, so we will not download them again
        """
        passed_files = [ ]
        url = config.REST_URL + '/log_lbconverter/doc'
        params = """{"select": ["id_file_orig"], "limit":null, "literal": "nm_base = '%s'"}""" % self.base
        req = requests.get(url, params={'$$':params}, headers=config.HEADERS)
        try:
            req.raise_for_status()
            response = req.json()
            passed_files = [ result['id_file_orig'] for result in response["results"] ]
        except:
            logger.error("""
                Erro ao tentar recuperar documentos que deram erro. Reposta: %s
            """ % req._content)
            raise Exception(req._content)
        return passed_files

    def  get_list_errors(self):
        erros_msg = set([])
        url = config.REST_URL + '/log_lbconverter/doc'
        params = """{"select":["error_msg"], "limit": null, "literal": "nm_base = '%s'"}""" % self.base
        req = requests.get(url, params={'$$':params}, headers=config.HEADERS)
        try:
            req.raise_for_status()
            response = req.json()
            erros_msg.update([ result["error_msg"] for result in response["results"] ])
        except Exception, e:
            logger.error("""
                Erro ao tentar recuperar mensagens de erro. Reposta: %s
            """ % req._content)
        return erros_msg


logger = logging.getLogger("LBConverter")
